package wind.android.bussiness.strategy.limitupseizeup.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.text.TextPaint
import android.util.AttributeSet
import android.view.View
import util.StringUtils
import wind.android.R
import kotlin.math.abs
import kotlin.math.max

/**
 * # **********************************************************************************************
 * # ClassName:      StrengthRectView
 * # Description:    矩形形状强度表示
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/10/18    10:59
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class StrengthRectView @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr) {
    /**
     * 矩形画笔
     */
    private val rectPaint = Paint()
    /**
     * 文字画笔
     */
    private val textPaint = TextPaint()
    /**
     * 正数颜色
     */
    private var rectPositiveColor: Int
    /**
     * 负数颜色
     */
    private var rectNegativeColor: Int
    /**
     * 矩形宽度
     */
    private var rectWidth: Int
    /**
     * 矩形高度
     */
    private var rectHeight: Int
    /**
     * 矩形距离文字的margin
     */
    private var textRectMargin: Int
    /**
     * 文字大小
     */
    private var textSize: Float
    /**
     * 强度值
     */
    private var strengthValue = 0.0
    /**
     * 最大强度值
     */
    private var maxStrengthValue = 10.0
    /**
     * 实心矩形占比
     */
    private var fillRectRate = 0.0

    init {
        val ta = context.obtainStyledAttributes(attrs, R.styleable.StrengthRectView)
        rectPositiveColor = ta.getColor(R.styleable.StrengthRectView_rect_positive_color, Color.RED)
        rectNegativeColor = ta.getColor(R.styleable.StrengthRectView_rect_negative_color, Color.RED)
        rectWidth = ta.getDimensionPixelSize(R.styleable.StrengthRectView_rect_width, 0)
        rectHeight = ta.getDimensionPixelSize(R.styleable.StrengthRectView_rect_height,
                StringUtils.dipToPx(5f))
        textRectMargin = ta.getDimensionPixelSize(R.styleable.StrengthRectView_rect_text_margin,
                StringUtils.dipToPx(3f))
        textSize = ta.getDimension(R.styleable.StrengthRectView_rect_txt_size,
                StringUtils.dipToPx(13f).toFloat())
        ta.recycle()
        //初始矩形画笔
        rectPaint.color = rectPositiveColor
        rectPaint.strokeWidth = StringUtils.dipToPx(1f).toFloat()
        rectPaint.isAntiAlias = true
        //初始化文字画笔
        textPaint.textSize = textSize
        textPaint.color = rectPositiveColor
        textPaint.isAntiAlias = true
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        val centerX = width / 2
        val centerY = height / 2
        if (strengthValue < 0) {
            rectPaint.color = rectNegativeColor
            textPaint.color = rectNegativeColor
        } else {
            rectPaint.color = rectPositiveColor
            textPaint.color = rectPositiveColor
        }
        rectPaint.style = Paint.Style.STROKE
        val rectLeft = if (rectWidth == 0) 0f else max(0f, (centerX - rectWidth / 2).toFloat())
        val rectTop = (centerY + textRectMargin / 2).toFloat()
        val rectRight = if (rectWidth == 0) width.toFloat() else (centerX + rectWidth / 2).toFloat()
        val rectBottom = rectTop + rectHeight
        canvas?.drawRect(rectLeft, rectTop, rectRight, rectBottom, rectPaint)
        rectPaint.style = Paint.Style.FILL
        if (fillRectRate != 0.0) {
            val fillRectRight = (rectLeft + (rectRight - rectLeft) * fillRectRate).toFloat()
            canvas?.drawRect(rectLeft, rectTop, fillRectRight, rectBottom, rectPaint)
        }
        val textWidth = textPaint.measureText(strengthValue.toString())
        canvas?.drawText(strengthValue.toString(), centerX - textWidth / 2,
                (centerY - textRectMargin / 2).toFloat(), textPaint)
    }

    /**
     * 设置强度值，并刷新页面
     *
     * @param value 强度值
     */
    @JvmOverloads
    fun setValue(value: Double, maxValue: Double = 10.0) {
        strengthValue = value
        maxStrengthValue = maxValue
        fillRectRate = if (value < maxValue) abs(value / maxValue) else 1.0
        invalidate()
    }
}