package wind.android.bussiness.strategy.limitupseizeup.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.text.TextPaint
import android.util.AttributeSet
import android.view.View
import util.StringUtils
import wind.android.R
import kotlin.math.abs
import kotlin.math.max

/**
 * # **********************************************************************************************
 * # ClassName:      StrengthSectionView
 * # Description:    矩形分段强度表示
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/10/18    10:59
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class StrengthSectionView @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr) {
    /**
     * 矩形画笔
     */
    private val rectPaint = Paint()
    /**
     * 文字画笔
     */
    private val textPaint = TextPaint()
    /**
     * 正数颜色
     */
    private var rectPositiveColor: Int
    /**
     * 负数颜色
     */
    private var rectNegativeColor: Int
    /**
     * 矩形宽度
     */
    private var rectWidth: Int
    /**
     * 矩形高度
     */
    private var rectHeight: Int
    /**
     * 矩形距离文字的margin
     */
    private var textRectMargin: Int
    /**
     * 文字大小
     */
    private var textSize: Float
    /**
     * 强度值
     */
    private var strengthValue = 0.0
    /**
     * 最大强度值
     */
    private var maxStrengthValue = 10.0
    /**
     * 是否显示 --
     */
    private var showDefaultTip = false
    /**
     * 是否反义设置-主力分歧
     */
    private var antis = false

    init {
        val ta = context.obtainStyledAttributes(attrs, R.styleable.StrengthSectionView)
        rectPositiveColor = ta.getColor(R.styleable.StrengthSectionView_section_positive_color, Color.RED)
        rectNegativeColor = ta.getColor(R.styleable.StrengthSectionView_section_negative_color, Color.RED)
        rectWidth = ta.getDimensionPixelSize(R.styleable.StrengthSectionView_section_width, 0)
        rectHeight = ta.getDimensionPixelSize(R.styleable.StrengthSectionView_section_height,
                StringUtils.dipToPx(5f))
        textRectMargin = ta.getDimensionPixelSize(R.styleable.StrengthSectionView_section_text_margin,
                StringUtils.dipToPx(3f))
        textSize = ta.getDimension(R.styleable.StrengthSectionView_section_txt_size,
                StringUtils.dipToPx(13f).toFloat())
        antis = ta.getBoolean(R.styleable.StrengthSectionView_section_antis, false)
        ta.recycle()
        //初始矩形画笔
        rectPaint.color = rectPositiveColor
        rectPaint.strokeWidth = StringUtils.dipToPx(1f).toFloat()
        rectPaint.isAntiAlias = true
        //初始化文字画笔
        textPaint.textSize = textSize
        textPaint.color = rectPositiveColor
        textPaint.isAntiAlias = true
    }

    fun setRectPositiveColor(color: Int) {
        rectPositiveColor = color
    }

    fun setRectNegativeColor(color: Int) {
        rectNegativeColor = color
    }

    fun setRectWidth(width: Int) {
        rectWidth = width
    }

    fun setRectHeight(height: Int) {
        rectHeight = height
    }

    fun setTextRectMargin(margin: Int) {
        textRectMargin = margin
    }

    fun setTextSize(size: Float) {
        textSize = size
    }

    fun setAntis(antis: Boolean) {
        this.antis = antis
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        canvas?.let {
            val centerX = width / 2
            val centerY = height / 2
            if (showDefaultTip) {
                val textWidth = textPaint.measureText("--")
                it.drawText("--", centerX - textWidth / 2,
                        (centerY - textRectMargin / 2).toFloat(), textPaint)
                return
            }
            if (strengthValue < 0 || antis) {
                rectPaint.color = rectNegativeColor
                textPaint.color = rectNegativeColor
            } else {
                rectPaint.color = rectPositiveColor
                textPaint.color = rectPositiveColor
            }
            rectPaint.style = Paint.Style.STROKE
            val rectLeft = if (rectWidth == 0) 0f else max(0f, (centerX - rectWidth / 2).toFloat())
            val rectTop = (centerY + textRectMargin / 2).toFloat()
            val rectRight = if (rectWidth == 0) width.toFloat() else (centerX + rectWidth / 2).toFloat()
            val rectBottom = rectTop + rectHeight
            it.drawRect(rectLeft, rectTop, rectRight, rectBottom, rectPaint)
            rectPaint.style = Paint.Style.FILL
            val showValue = 10 * strengthValue / maxStrengthValue
            val fillWidth = abs((rectRight - rectLeft) * strengthValue / maxStrengthValue).toFloat()
            val perWidth = (rectRight - rectLeft) / 5
            if (fillWidth > 0) {
                val count = (fillWidth / perWidth).toInt()
                var diff = 0
                for (index in 0..count) {
                    diff += 10 * if (index < 1) 0 else index - 1
                    rectPaint.alpha = 115 + 50 * index - diff
                    if (index == count) {
                        it.drawRect(rectLeft + perWidth * index, rectTop, rectLeft + fillWidth, rectBottom, rectPaint)
                    } else {
                        it.drawRect(rectLeft + perWidth * index, rectTop, rectLeft + perWidth * (index + 1), rectBottom, rectPaint)
                    }
                }
            }
            val textWidth = textPaint.measureText(showValue.toString())
            it.drawText(showValue.toString(), centerX - textWidth / 2,
                    (centerY - textRectMargin / 2).toFloat(), textPaint)
        }
    }

    /**
     * 设置强度值，并刷新页面
     *
     * @param value 强度值
     */
    @JvmOverloads
    fun setValue(value: Double, maxValue: Double = 10.0) {
        strengthValue = value
        maxStrengthValue = maxValue
        showDefaultTip = false
        invalidate()
    }

    fun setDefaultTip() {
        showDefaultTip = true
        invalidate()
    }
}