package wind.android.bussiness.strategy.limitupseizeup.view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.Path
import android.text.TextPaint
import android.util.AttributeSet
import android.view.View
import util.StringUtils
import util.ThemeUtils
import wind.android.R
import kotlin.math.max
import kotlin.math.min
import kotlin.math.sqrt

/**
 * # **********************************************************************************************
 * # ClassName:      StrengthTriangleView
 * # Description:    三角形形状强度表示
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2019/10/18    15:42
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class StrengthTriangleView @JvmOverloads constructor(
        context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr) {
    /**
     * 三角形画笔
     */
    private val trianglePaint = Paint()
    /**
     * 文字画笔
     */
    private val textPaint = TextPaint()
    /**
     * path 用于画三角形
     */
    private val path = Path()
    /**
     * 颜色
     */
    private var triangleColor: Int
    /**
     * “--” 颜色
     */
    private var defaultTipColor: Int
    /**
     * 直角三角形底部宽度
     */
    private var triangleWidth: Int
    /**
     * 直角三角形高度
     */
    private var triangleHeight: Int
    /**
     * 三角形距离文字的margin
     */
    private var textTriangleMargin: Int
    /**
     * 文字大小
     */
    private var textSize: Float
    /**
     * 强度值
     */
    private var strengthValue = 0.0
    /**
     * 最大强度值
     */
    private var maxStrengthValue = 10.0
    /**
     * 实心三角形占比
     */
    private var fillTriangleRate = 0.0
    /**
     * 是否显示 --
     */
    private var showDefaultTip = false

    init {
        val ta = context.obtainStyledAttributes(attrs, R.styleable.StrengthTriangleView)
        triangleColor = ta.getColor(R.styleable.StrengthTriangleView_triangle_color, Color.RED)
        triangleWidth = ta.getDimensionPixelSize(R.styleable.StrengthTriangleView_triangle_width, 0)
        triangleHeight = ta.getDimensionPixelSize(R.styleable.StrengthTriangleView_triangle_height, 0)
        textTriangleMargin = ta.getDimensionPixelSize(R.styleable.StrengthTriangleView_triangle_text_margin,
                TEXT_TRIANGLE_DEFAULT_MARGIN)
        textSize = ta.getDimension(R.styleable.StrengthTriangleView_triangle_txt_size,
                StringUtils.dipToPx(13f).toFloat())
        ta.recycle()
        //初始矩形画笔
        trianglePaint.color = triangleColor
        trianglePaint.strokeWidth = StringUtils.dipToPx(1f).toFloat()
        trianglePaint.isAntiAlias = true
        //初始化文字画笔
        textPaint.textSize = textSize
        textPaint.color = triangleColor
        textPaint.isAntiAlias = true
        defaultTipColor = ThemeUtils.getColor(resources, R.color.txt_color_black_1, R.color.txt_color_white_1)
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        val centerX = width / 2
        val centerY = height / 2
        val triangleHeight = if (triangleHeight == 0) measuredHeight.toFloat() else
            min(measuredHeight.toFloat(), triangleHeight.toFloat())
        val rectLeft = if (triangleWidth == 0) 0f else max(0f, (centerX - triangleWidth / 2).toFloat())
        val rectTop = centerY - triangleHeight / 2
        val rectRight = if (triangleWidth == 0) width.toFloat() else (centerX + triangleWidth / 2).toFloat()
        val rectBottom = centerY + triangleHeight / 2
        canvas?.let {
            if (!showDefaultTip) {
                trianglePaint.style = Paint.Style.STROKE
                drawTriangle(rectLeft, rectTop, rectRight, rectBottom, trianglePaint, path, it)
                if (fillTriangleRate != 0.0) {
                    val fillRectRight = (rectLeft + (rectRight - rectLeft) * fillTriangleRate).toFloat()
                    val fillRectTop = (rectBottom - triangleHeight * fillTriangleRate).toFloat()
                    trianglePaint.style = Paint.Style.FILL
                    drawTriangle(rectLeft, fillRectTop, fillRectRight, rectBottom, trianglePaint, path, it)
                }
                textPaint.textSize = textSize
                textPaint.color = triangleColor
                val textLeft = (rectLeft + (rectRight - rectLeft) * TEXT_TRIANGLE_RATE).toFloat() -
                        textTriangleMargin - textPaint.measureText(strengthValue.toString())
                val textBottom = (rectBottom - triangleHeight * TEXT_TRIANGLE_RATE).toFloat()
                it.drawText(strengthValue.toString(), textLeft, textBottom, textPaint)
            } else {
                textPaint.textSize = StringUtils.dipToPx(16f).toFloat()
                textPaint.color = defaultTipColor
                val text = "--"
                val textLeft = centerX - textPaint.measureText(text) / 2
                val fm = textPaint.fontMetrics
                //执行：getHeight()/2-fm.descent , 将整个文字区域抬高至控件的1/2
                //执行： + (fm.bottom - fm.top) / 2 , 将文本下沉文本高度的一半
                val textBottom = centerY - fm.descent + (fm.bottom - fm.top) / 2
                it.drawText(text, textLeft, textBottom, textPaint)
            }
        }
    }

    /**
     * 画直角三角形
     *
     * @param left 左边界
     * @param top 上边界
     * @param right 右边界
     * @param bottom 下边界
     * @param paint 画笔
     * @param path path
     * @param canvas 画布
     */
    private fun drawTriangle(left: Float, top: Float, right: Float, bottom: Float,
                             paint: Paint, path: Path, canvas: Canvas) {
        path.reset()
        path.moveTo(left, bottom)
        path.lineTo(right, bottom)
        path.lineTo(right, top)
        path.close()
        canvas.drawPath(path, paint)
    }

    /**
     * 设置强度值，并刷新页面
     *
     * @param value 强度值
     */
    @JvmOverloads
    fun setValue(value: Double, maxValue: Double = 10.0) {
        strengthValue = value
        maxStrengthValue = maxValue
        fillTriangleRate = if (value < maxValue) sqrt(value / maxValue) else 1.0
        showDefaultTip = false
        invalidate()
    }

    fun setDefaultTip() {
        showDefaultTip = true
        invalidate()
    }

    companion object ConstValue {
        const val TEXT_TRIANGLE_RATE = 0.8
        @JvmStatic
        val TEXT_TRIANGLE_DEFAULT_MARGIN = StringUtils.dipToPx(5f)
    }
}