package wind.android.f5.bussiness.news.presenter

import android.annotation.SuppressLint
import com.alibaba.fastjson.JSON
import log.LogHelper
import net.activity.BaseHandle
import net.data.network.SkyCallbackData
import net.datamodel.network.TimeCalculate
import net.datamodel.speed.WindCodeType
import net.listener.ISkyDataListenerEx
import net.protocol.model.Error
import net.protocol.model.Token
import net.util.TimeHelper
import util.NewsTimeAssist
import wind.android.f5.bussiness.news.view.ISpeedNewsView
import wind.android.f5.model.business.SkySubject
import wind.android.f5.net.base.SpeedNewsConnection
import wind.android.f5.bussiness.investor.model.InvestorHitsModel
import wind.android.f5.bussiness.investor.model.InvestorQaRequestModel
import wind.android.f5.manager.DragonCallBack
import wind.android.f5.manager.DragonManager
import wind.android.news.anews.NewsTitleModel
import wind.android.news.anews.SkyNewsResponse
import java.text.SimpleDateFormat
import java.util.*
import java.util.regex.Pattern

/**
 * # ***********************************************************************************************
 * # ClassName:      SpeedF5NewsPresenter
 * # Function:       F5-新闻列表请求类
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/1/22$    11:36$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class SpeedF5NewsPresenter(var newsView: ISpeedNewsView? = null) {

    companion object {
        const val PAGE_SIZE = 25
    }

    fun onDestoryView() {
        newsView = null
    }

    /**
     * 请求新闻列表
     */
    fun requestNews(windCode: String?, pageIndex: Int) {
        if (windCode.isNullOrBlank()) {
            stopRefresh(pageIndex)
            return
        }
        SpeedNewsConnection.getInstance().startRequestNews(WindCodeType.getWindSecType(windCode), windCode, pageIndex, PAGE_SIZE,
                object : SpeedNewsConnection.OnDataReceivedListener<MutableList<NewsTitleModel>> {
                    override fun onDateReceived(t: MutableList<NewsTitleModel>?) {
                        try {
                            t?.let {
                                if (pageIndex == 1) {
                                    newsView?.setNewsList(it)
                                    requestInvestorQa(windCode)
                                } else {
                                    newsView?.updateNewsList(it)
                                }
                                return
                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }
                        stopRefresh(pageIndex)
                    }

                    override fun onError(e: Error?, mToken: Token<*>?, mActivity: BaseHandle?, errorData: Any?) {
                        LogHelper.print("SpeedF5NewsPresenter", "onError size:${e?.name}")
                        stopRefresh(pageIndex)
                    }
                }, false)
    }

    private fun stopRefresh(pageIndex: Int) {
        if (pageIndex > 1) {
            newsView?.loadMoreComplete()
        }
    }

    /**
     * 请求投资者问答
     */
    private fun requestInvestorQa(windCode: String?) {
        if (windCode.isNullOrBlank()) {
            return
        }
        val endDate = TimeHelper.getInstance().getTimeEx(System.currentTimeMillis(), NewsTimeAssist.OCEAN_TIME_FMT_STR)
        val c = Calendar.getInstance()
        // 请求一个月内的投资者问答数据
        c.add(Calendar.MONTH, -1)
        val startDate = TimeHelper.getInstance().getTimeEx(c.timeInMillis, NewsTimeAssist.OCEAN_TIME_FMT_STR)
        val model = InvestorQaRequestModel()
        model.indexName = "investor_qa"
        model.dimension = windCode
        model.pageIndex = 1
        model.pageNum = PAGE_SIZE
        model.fileds = "unit,unitinfo"
        model.hfields = "unit,unitinfo"
        SkySubject.requestInvestorQa(model, startDate, endDate, object : ISkyDataListenerEx {
            override fun OnSkyCallback(data: SkyCallbackData?) {
                try {
                    val elementAt = data?.data?.elementAtOrNull(0)
                    if (elementAt is SkyNewsResponse) {
                        val source = String(elementAt.a_bytes)
                        val sourceObject = JSON.parseObject(source) ?: null
                        sourceObject?.let {
                            val hits = it.getJSONObject("hits")
                            if (hits != null) {
                                val s = hits.toJSONString()
                                val investorHitsModel = JSON.parseObject(s, InvestorHitsModel::class.java) ?: null
                                val investorQaModel = investorHitsModel?.hits?.elementAtOrNull(0)?._source
                                investorQaModel?.let {
                                    model ->
                                    val newsTitleModel = NewsTitleModel()
                                    newsTitleModel.isInvestor = true
                                    newsTitleModel.title =  "投资者问答：" + model.getUnitValue()
                                    newsTitleModel.newsTime = model.getPublishDateValue()
                                    newsView?.updateInvestor(newsTitleModel)
                                }
                            }
                        }
                    }
                    requestTopTenData(windCode)
                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }

            override fun OnSkyError(code: Int, serialNum: Int) {
                LogHelper.print("SpeedNewsManager", "OnSkyError code:$code")
                requestTopTenData(windCode)
            }
        })
    }

    /**
     * 请求龙虎榜
     */
    private fun requestTopTenData(windCode: String) {
        val newDate = TimeCalculate.datePlus(TimeHelper.getInstance().curDate, -6)
        val startDate = TimeCalculate.convDate2Str(newDate, "yyyyMMdd")
        val endDate = TimeHelper.getInstance().date
        DragonManager.getInstance().queryStock(windCode, startDate, endDate, object : DragonCallBack {
            override fun onError(e: Error?, mToken: Token<*>?, mActivity: BaseHandle?, errorData: Any?) {

            }

            @SuppressLint("SimpleDateFormat")
            override fun callBack(s: String?) {
                try {
                    val p = Pattern.compile("(\\d+)")
                    val m = p.matcher(s ?: "")
                    if (m.find()) {
                        val date = m.group(1) ?: ""
                        if (date.isNotEmpty()) {
                            val dateFormatOld = SimpleDateFormat("yyyy-MM-dd")
                            val dateFormatNew = SimpleDateFormat("yyyyMMdd")
                            val dateNew = dateFormatOld.format(dateFormatNew.parse(date)) ?: ""
                            getDragonData(dateNew, windCode)
                        }
                    } else {
                        LogHelper.print("StockAdapter", "该股票最近一周未上龙虎榜")
                    }
                } catch (ex: Exception) {
                    LogHelper.print("StockAdapter", "日期返回格式错误")
                }
            }
        })
    }

    private fun getDragonData(date: String, windCode: String) {
        val newsTitleModel = NewsTitleModel()
        newsTitleModel.newsId = "-100"
        newsTitleModel.newsTime = "00:01"
        newsTitleModel.isInDragonList = true
        newsTitleModel.title = (WindCodeType.getStockNameQuick(windCode) ?: "") + " "+ date + " 上龙虎榜"
        newsView?.updateDragonData(newsTitleModel)
    }
}