package wind.android.f5.level2

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.RectF
import android.util.AttributeSet
import android.view.View
import net.datamodel.speed.MoneyGameDataItem
import session.F5Session
import util.StringUtils
import util.ThemeUtils
import wind.android.f5.model.business.Tool
import wind.android.news.anews.StockUtil

/**
 * # ***********************************************************************************************
 * # ClassName:      F5MoneyFlowHistogramView$
 * # Function:       单券-实时资金流流向-柱状图
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/2/24$    14:08$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class F5MoneyFlowHistogramView @JvmOverloads constructor(context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0) :
        View(context, attrs, defStyleAttr){

    companion object {

        const val BUY_ONE_HUNDRED = 100

        const val BUY_ONE_MILLION = 1000000
    }

    private val mDescList by lazy { listOf("大于100万", "20-100万", "4-20万", "小于4万") }

    private val mDescColor by lazy { ThemeUtils.getColor(Color.WHITE, Color.BLACK) }

    private val mLineColor by lazy { ThemeUtils.getColor(-0xd0d0d1, -0x2e2e2f) }

    private val mRightColor by lazy { Color.parseColor("#ff0000") }

    private val mLeftColor by lazy { Color.parseColor("#66cdaa") }

    private val mMiniSpace by lazy { StringUtils.dipToPx(2f).toFloat() }

    private val mSpace by lazy { StringUtils.dipToPx(5f).toFloat() }

    private val mEdgeSpace by lazy { StringUtils.dipToPx(45f).toFloat() }

    private var mHeight = 0f

    private var mWidth = 0f

    private var mData: MoneyGameDataItem? = null

    private val mLinePaint by lazy { Paint() }
    private val mVolumePaint by lazy { Paint() }
    private val mHistogramPaint by lazy { Paint() }
    private val mDescPaint by lazy { Paint() }

    private var mMaxVolume = 0f

    private var mPerVolumeWidth = 0f

    private val mHistogramHeight by lazy { StringUtils.dipToPx(15f).toFloat() }

    private var mMiddleX = 0f

    init {
        mDescPaint.textSize = StringUtils.dipToPx(10f).toFloat()
        mDescPaint.color = mDescColor
        mDescPaint.typeface = F5Session.getInstance().getDinTypeface(getContext())
        mVolumePaint.textSize = StringUtils.dipToPx(10f).toFloat()
        mVolumePaint.typeface = F5Session.getInstance().getDinTypeface(getContext())
    }

    fun setData(data: MoneyGameDataItem) {
        if (data.volume1 == java.lang.Float.MAX_VALUE) {
            data.volume1 = 0f
        }
        if (data.volume2 == java.lang.Float.MAX_VALUE) {
            data.volume2 = 0f
        }
        if (data.volume3 == java.lang.Float.MAX_VALUE) {
            data.volume3 = 0f
        }
        if (data.volume4 == java.lang.Float.MAX_VALUE) {
            data.volume4 = 0f
        }
        mData = data
        mMaxVolume = 0f
        mMaxVolume = Math.max(mMaxVolume, Math.abs(data.volume1))
        mMaxVolume = Math.max(mMaxVolume, Math.abs(data.volume2))
        mMaxVolume = Math.max(mMaxVolume, Math.abs(data.volume3))
        mMaxVolume = Math.max(mMaxVolume, Math.abs(data.volume4))
        invalidate()
    }

    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
        val height = (mHistogramHeight * 4 + mSpace * 2 + mMiniSpace * 5).toInt()
        val specWidth = View.MeasureSpec.makeMeasureSpec(measuredWidth, View.MeasureSpec.EXACTLY)
        val specHeight = View.MeasureSpec.makeMeasureSpec(height, View.MeasureSpec.EXACTLY)
        setMeasuredDimension(specWidth, specHeight)
    }

    override fun onLayout(changed: Boolean, left: Int, top: Int, right: Int, bottom: Int) {
        super.onLayout(changed, left, top, right, bottom)
        mWidth = measuredWidth.toFloat()
        mHeight = measuredHeight.toFloat()
        mMiddleX = mWidth / 2f
    }

    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        drawLine(canvas)
        if (Math.abs(mMaxVolume) > 0f) {
            mPerVolumeWidth = (mWidth / 2 - mMiniSpace - mSpace * 2 - mEdgeSpace) / mMaxVolume
        }
        if (mPerVolumeWidth == 0f) {
            drawRightHistogram(canvas, 0, 0f)
            drawRightHistogram(canvas, 1, 0f)
            drawRightHistogram(canvas, 2, 0f)
            drawRightHistogram(canvas, 3, 0f)
        } else {
            drawHistogram(canvas, 0, mData?.volume1 ?: 0f)
            drawHistogram(canvas, 1, mData?.volume2 ?: 0f)
            drawHistogram(canvas, 2, mData?.volume3 ?: 0f)
            drawHistogram(canvas, 3, mData?.volume4 ?: 0f)
        }
    }

    private fun drawLine(canvas: Canvas?) {
        mLinePaint.color = mLineColor
        val lineWidth = StringUtils.dipToPx(1f).toFloat()
        mLinePaint.strokeWidth = lineWidth
        mLinePaint.isAntiAlias = false
        mLinePaint.style = Paint.Style.STROKE
        canvas?.drawLine((mWidth - lineWidth) / 2, mSpace,(mWidth + lineWidth) / 2,
                mHeight - mSpace, mLinePaint)
    }

    private fun drawHistogram(canvas: Canvas?, index: Int, volume: Float) {
        if (volume >= 0f) {
            drawRightHistogram(canvas, index, volume)
        } else {
            drawLeftHistogram(canvas, index, volume)
        }
    }

    private fun drawLeftHistogram(canvas: Canvas?, index: Int, volume: Float) {
        val startY = mSpace + (mHistogramHeight + mMiniSpace) * index + mMiniSpace
        val endX = mMiddleX - mMiniSpace
        val histogramWidth = mPerVolumeWidth * Math.abs(volume)
        val startX = endX - histogramWidth
        val descMetrics = mDescPaint.fontMetrics
        val descTextY = (descMetrics.bottom - descMetrics.top) / 2 - descMetrics.bottom
        val desc = mDescList.elementAtOrNull(index) ?: ""
        canvas?.drawText(desc, mMiddleX + mSpace, startY + mHistogramHeight / 2 + descTextY, mDescPaint)
        mHistogramPaint.color = mLeftColor
        mHistogramPaint.style = Paint.Style.FILL
        mHistogramPaint.isAntiAlias = true
        val rect = RectF()
        rect.set(startX, startY, endX, startY + mHistogramHeight)
        val round = mHistogramHeight / 2
        canvas?.drawRoundRect(rect, round, round, mHistogramPaint )
        val volumeMetrics = mVolumePaint.fontMetrics
        val volumeTextY = (volumeMetrics.bottom - volumeMetrics.top) / 2 - volumeMetrics.bottom
        val volumeStr = getVolumeStr(volume)
        mVolumePaint.color = getVolumeColor(volume)
        canvas?.drawText(volumeStr, startX - mSpace - mVolumePaint.measureText(volumeStr),
                startY + mHistogramHeight / 2 + volumeTextY, mVolumePaint)
    }

    private fun drawRightHistogram(canvas: Canvas?, index: Int, volume: Float) {
        val startY = mSpace + (mHistogramHeight + mMiniSpace) * index + mMiniSpace
        val startX = mMiddleX + mMiniSpace
        val histogramWidth = mPerVolumeWidth * volume
        val endX = startX + histogramWidth
        val descMetrics = mDescPaint.fontMetrics
        val descTextY = (descMetrics.bottom - descMetrics.top) / 2 - descMetrics.bottom
        val desc = mDescList.elementAtOrNull(index) ?: ""
        canvas?.drawText(desc, mMiddleX - mDescPaint.measureText(desc) - mSpace, startY + mHistogramHeight / 2 + descTextY, mDescPaint)
        mHistogramPaint.color = mRightColor
        mHistogramPaint.style = Paint.Style.FILL
        mHistogramPaint.isAntiAlias = true
        val rect = RectF()
        rect.set(startX, startY, endX, startY + mHistogramHeight)
        val round = mHistogramHeight / 2
        canvas?.drawRoundRect(rect, round, round, mHistogramPaint )
        val volumeMetrics = mVolumePaint.fontMetrics
        val volumeTextY = (volumeMetrics.bottom - volumeMetrics.top) / 2 - volumeMetrics.bottom
        val volumeStr = getVolumeStr(volume)
        mVolumePaint.color = getVolumeColor(volume)
        canvas?.drawText(volumeStr, endX + mSpace, startY + mHistogramHeight / 2 + volumeTextY, mVolumePaint)
    }

    private fun getVolumeStr(volume: Float): String {
        return when {
            volume == 0f -> "--"
            Math.abs(volume) < BUY_ONE_MILLION -> Tool.numberFormatAndRound(volume / BUY_ONE_HUNDRED, 0).toString() + "手"
            else -> Tool.numberFormatAndRound(volume / BUY_ONE_MILLION, 2) + "万手"
        }
    }

    private fun getVolumeColor(volume: Float): Int {
        return StockUtil.getChangeColor(volume)
    }
}