package wind.android.f5.view.bottom.presenter

import android.os.Message
import base.ActivityHandler
import net.bussiness.Stock
import net.datamodel.network.Indicator
import net.datamodel.network.RealQuoteData
import net.datamodel.network.RealQuoteItem
import net.listener.ITcpDataReceiver
import net.network.model.SpeedErrorModel
import net.network.speed.TcpProcessor
import network.NetManager
import wind.android.f5.model.IndexComponetModel
import wind.android.f5.view.bottom.presenter.F5IndexComponentPresenter.F5IndexListDataReqType.*
import wind.android.f5.view.bottom.subview.index.IF5IndexSubView
import java.lang.ref.WeakReference
import java.util.*

/**
 * # **********************************************************************************************
 * # ClassName:      F5IndexComponentPresenter
 * # Description:    大盘指数成分TAB和板块TAB presenter层代码
 * # Author:         wplin
 * # Version:        Ver 1.0
 * # **********************************************************************************************
 * # Modified By:    wplin     2020/2/25    16:27
 * # Modifications:  initial
 * # **********************************************************************************************
 */

class F5IndexComponentPresenter(iView: IF5IndexSubView<IndexComponetModel>) : ActivityHandler.ActivityHandlerListener {
    /**
     * view层弱引用
     */
    private val weakView = WeakReference<IF5IndexSubView<IndexComponetModel>>(iView)
    /**
     * 记录订阅的windCode
     */
    private var subWindCodes: Array<String>? = null
    /**
     * 记录列表中的数据，用于查找
     */
    private var copyList: List<IndexComponetModel>? = null
    /**
     * 板块TAB订阅指标
     */
    private val plateIndicator = intArrayOf(Indicator.DI_SHORTNAMECHS, Indicator.DI_CHANGE,
            Indicator.DI_CHANGERANGE, Indicator.DI_CHANGE5MINS)
    /**
     * 成分TAB订阅指标
     */
    private val componentIndicator = intArrayOf(Indicator.DI_SHORTNAMECHS, Indicator.DI_NEWPRICE,
            Indicator.DI_CHANGE, Indicator.DI_CHANGERANGE,
            Indicator.DI_CHANGEHANDRATE, Indicator.DI_CAPITALMARKETVALUE)
    /**
     * 订阅指标
     */
    private var subIndicator: IntArray = componentIndicator

    private val receiver: ITcpDataReceiver by lazy {
        object : ITcpDataReceiver {
            override fun getTimeOutValue() = 0

            override fun onSubDataRecived(items: Any?) {
                //post到主线程便于copyList查找，避免多线程问题
                if (items is Vector<*>) {
                    val message = Message.obtain()
                    message.what = MSG_UPDATE_LIST_POS
                    message.obj = items
                    sendToMainThread(message)
                }
            }

            override fun onErrorReceived(model: SpeedErrorModel?, type: Int) {
            }

            override fun onMaskDataReceived(requestID: Int, data: Any?, type: Int): Boolean {
                if (data is Vector<*>) {
                    val realQuoteItems = data as? Vector<RealQuoteItem>
                    val list = arrayListOf<IndexComponetModel>()
                    realQuoteItems?.forEach {
                        val item = IndexComponetModel()
                        item.stockName = it.StockName
                        item.windCode = it.WindCode
                        parseSubData(item, it)
                        list.add(item)
                    }
                    //post到主线程便于copyList查找，避免多线程问题
                    val message = Message.obtain()
                    message.what = MSG_UPDATE_LIST_ALL
                    message.obj = list
                    sendToMainThread(message)
                    return true
                }
                return false
            }
        }
    }

    /**
     * 解析订阅指标
     */
    private fun parseSubData(item: IndexComponetModel, realQuoteItem: RealQuoteItem?) {
        if (realQuoteItem != null) {
            realQuoteItem.indicators?.forEachIndexed { index, i ->
                when (i) {
                    Indicator.DI_NEWPRICE -> item.price = realQuoteItem.value[index]
                    Indicator.DI_CHANGE -> item.change = realQuoteItem.value[index]
                    Indicator.DI_CHANGERANGE -> item.changeRange = realQuoteItem.value[index]
                    Indicator.DI_CHANGEHANDRATE -> item.changeHandRate = realQuoteItem.value[index]
                    Indicator.DI_CAPITALMARKETVALUE -> item.totalPrice = realQuoteItem.value[index]
                    Indicator.DI_CHANGE5MINS -> item.change5Min = realQuoteItem.value[index]
                }
            }
        }
    }

    /**
     * 解除与view层的关联
     */
    fun detachView() {
        weakView.clear()
    }

    /**
     * 请求列表数据，通过speed排序接口获得
     *
     * @param windCode
     * @param type
     */
    fun reqListData(windCode: String, type: F5IndexListDataReqType) {
        unSub()
        val view = weakView.get() ?: return
        view.updateListAll(null)
        subWindCodes = null
        copyList = null
        val sectorName = getSectorName(windCode, type)
        val pageSize = when (type) {
            COMPONENT_RISE_TYPE, COMPONENT_FALL_TYPE,
            COMPONENT_CHANGE_HAND_TYPE, COMPONENT_MARKET_VALUE_TYPE -> 10
            PLATE_RISE_TYPE, PLATE_FALL_TYPE,
            PLATE_RISE_5_TYPE, PLATE_FALL_5_TYPE -> 5
        }
        val indicator = when (type) {
            COMPONENT_RISE_TYPE, COMPONENT_FALL_TYPE,
            PLATE_RISE_TYPE, PLATE_FALL_TYPE -> Indicator.DI_CHANGERANGE
            COMPONENT_CHANGE_HAND_TYPE -> Indicator.DI_CHANGEHANDRATE
            COMPONENT_MARKET_VALUE_TYPE -> Indicator.DI_CAPITALMARKETVALUE
            PLATE_RISE_5_TYPE, PLATE_FALL_5_TYPE -> Indicator.DI_CHANGE5MINS
        }
        val sortDirection = when (type) {
            COMPONENT_FALL_TYPE, PLATE_FALL_TYPE, PLATE_FALL_5_TYPE -> 2
            else -> 1
        }
        val reqId = when (type) {
            COMPONENT_RISE_TYPE -> REQ_COMPONENT_RISE_ID
            COMPONENT_FALL_TYPE -> REQ_COMPONENT_FALL_ID
            COMPONENT_CHANGE_HAND_TYPE -> REQ_COMPONENT_CHANGE_HAND_ID
            COMPONENT_MARKET_VALUE_TYPE -> REQ_COMPONENT_MARKET_VALUE_ID
            PLATE_RISE_TYPE -> REQ_PLATE_RISE_ID
            PLATE_FALL_TYPE -> REQ_PLATE_FALL_ID
            PLATE_RISE_5_TYPE -> REQ_PLATE_RISE_5MIN_ID
            PLATE_FALL_5_TYPE -> REQ_PLATE_FALL_5MIN_ID
        }
        subIndicator = when (type) {
            COMPONENT_RISE_TYPE, COMPONENT_FALL_TYPE,
            COMPONENT_CHANGE_HAND_TYPE, COMPONENT_MARKET_VALUE_TYPE -> componentIndicator
            PLATE_RISE_TYPE, PLATE_FALL_TYPE,
            PLATE_RISE_5_TYPE, PLATE_FALL_5_TYPE -> plateIndicator
        }
        if (NetManager.isNetworkAvailable()) {
            view.showProgress()
        }
        Stock.RankListRequest(sectorName, pageSize, 0, indicator, sortDirection,
                object : ITcpDataReceiver {
                    override fun getTimeOutValue() = 0

                    override fun onSubDataRecived(items: Any?) {
                    }

                    override fun onErrorReceived(model: SpeedErrorModel?, type: Int) {
                        weakView.get()?.hideProgress()
                    }

                    override fun onMaskDataReceived(requestID: Int, data: Any?, type: Int): Boolean {
                        val v = weakView.get() ?: return false
                        v.hideProgress()
                        if (data is RealQuoteData && data.receiveId == reqId) {
                            val list = arrayListOf<String>()
                            data.RealQuoteList?.forEach {
                                val realQuoteItem = it as RealQuoteItem
                                list.add(realQuoteItem.WindCode)
                            }
                            val message = Message.obtain()
                            message.what = MSG_SUB_LIST
                            message.obj = list
                            sendToMainThread(message)
                            return true
                        }
                        return false
                    }

                }, reqId)
    }

    /**
     * 获取speed 排序接口请求参数sectorName
     */
    private fun getSectorName(windCode: String, type: F5IndexListDataReqType): String {
        return when (type) {
            COMPONENT_RISE_TYPE, COMPONENT_FALL_TYPE,
            COMPONENT_CHANGE_HAND_TYPE, COMPONENT_MARKET_VALUE_TYPE -> {
                when (windCode) {
                    "399001.SZ" -> SZ_COMPONENT_SECTOR
                    "399006.SZ" -> CY_COMPONENT_SECTOR
                    else -> SH_COMPONENT_SECTOR
                }
            }
            PLATE_RISE_TYPE, PLATE_FALL_TYPE,
            PLATE_RISE_5_TYPE, PLATE_FALL_5_TYPE -> {
                SH_PLATE_SECTOR
            }
        }
    }

    /**
     * post到主线程
     */
    private fun sendToMainThread(msg: Message) {
        ActivityHandler.getInstance(this).sendMessage(msg)
    }

    /**
     * 订阅
     *
     * @param subCodes 订阅的windCode数组
     * @param unSubCodes 反订阅的windCode数组
     */
    fun sub(subCodes: Array<String>? = subWindCodes, unSubCodes: Array<String>? = null) {
        if (subCodes == null) {
            return
        }
        TcpProcessor.getInstance().registerPushDataReceiver(receiver)
        Stock.SubscribeRequest(subCodes, unSubCodes, subIndicator, receiver)
        subWindCodes = subCodes
    }

    /**
     * 反订阅
     */
    fun unSub() {
        Stock.SubscribeRequest(null, subWindCodes, subIndicator, receiver)
        TcpProcessor.getInstance().unregisterPushDataReceiver(receiver)
    }

    /**
     * speed断网重连
     */
    fun speedReconnect(windCode: String, type: F5IndexListDataReqType){
        if (copyList == null) {
            reqListData(windCode, type)
        } else {
            sub()
        }
    }

    override fun handleMessage(msg: Message?) {
        when (msg?.what) {
            MSG_SUB_LIST -> {
                val windCodes = (msg.obj as ArrayList<String>).toTypedArray()
                sub(windCodes, subWindCodes)
            }
            MSG_UPDATE_LIST_ALL -> {
                val list = msg.obj as ArrayList<IndexComponetModel>
                copyList = list
                weakView.get()?.updateListAll(list)
            }
            MSG_UPDATE_LIST_POS -> {
                val items = msg.obj as? Vector<RealQuoteItem> ?: return
                for (item in items) {
                    if (item == null) {
                        continue
                    }
                    val indexOfFirst = copyList?.indexOfFirst {
                        it.windCode == item.WindCode
                    } ?: -1
                    val model = copyList?.elementAtOrNull(indexOfFirst)
                    model?.let {
                        parseSubData(it, item)
                        weakView.get()?.updateListByPosition(indexOfFirst)
                    }
                }
            }
        }
    }

    /**
     * 请求类型
     */
    enum class F5IndexListDataReqType {
        COMPONENT_RISE_TYPE, COMPONENT_FALL_TYPE,
        COMPONENT_CHANGE_HAND_TYPE, COMPONENT_MARKET_VALUE_TYPE,
        PLATE_RISE_TYPE, PLATE_FALL_TYPE,
        PLATE_RISE_5_TYPE, PLATE_FALL_5_TYPE
    }

    companion object {
        /**
         * 上证A股 板块id  指数板块
         */
        private const val SH_PLATE_SECTOR = "a39901012c000000"
        /**
         * 深证A股 板块id 个股板块
         */
        private const val SZ_PLATE_SECTOR = "a001010300000000"
        /**
         * 上证 成分板块id  指数板块
         */
        private const val SH_COMPONENT_SECTOR = "a001030101000000"
        /**
         * 深证 成分板块id  指数板块
         */
        private const val SZ_COMPONENT_SECTOR = "1000000088000000"
        /**
         * 创业板 成分板块id  指数板块
         */
        private const val CY_COMPONENT_SECTOR = "1000000092000000"
        /**
         * 板块涨幅榜
         */
        private const val REQ_PLATE_RISE_ID = 0x101
        /**
         * 板块跌幅榜
         */
        private const val REQ_PLATE_FALL_ID = 0x102
        /**
         * 板块5分钟涨速
         */
        private const val REQ_PLATE_RISE_5MIN_ID = 0x103
        /**
         * 板块5分钟跌速
         */
        private const val REQ_PLATE_FALL_5MIN_ID = 0x104
        /**
         * 成分涨幅榜
         */
        private const val REQ_COMPONENT_RISE_ID = 0x105
        /**
         * 成分跌幅榜
         */
        private const val REQ_COMPONENT_FALL_ID = 0x106
        /**
         * 成分换手率
         */
        private const val REQ_COMPONENT_CHANGE_HAND_ID = 0x107
        /**
         * 成分总市值
         */
        private const val REQ_COMPONENT_MARKET_VALUE_ID = 0x108
        /**
         * handler消息码
         */
        private const val MSG_SUB_LIST = 0x11
        private const val MSG_UPDATE_LIST_ALL = 0x12
        private const val MSG_UPDATE_LIST_POS = 0x13
        /**
         * 标识TAB
         */
        const val DATA_TAB0 = 0
        const val DATA_TAB1 = 1
        const val DATA_TAB2 = 2
        const val DATA_TAB3 = 3
    }
}