package wind.android.market.business.option.presenter

import android.os.Message
import base.ActivityHandler
import net.bussiness.Stock
import net.data.network.SkyCallbackData
import net.datamodel.network.CommonFunc
import net.datamodel.network.Indicator
import net.datamodel.network.RealQuoteItem
import net.datamodel.network.SpeedConst
import net.listener.ISkyDataListenerEx
import net.listener.ITcpDataReceiver
import net.network.model.SpeedErrorModel
import net.network.speed.TcpProcessor
import net.util.Assist
import wind.android.market.business.option.manager.OptionManager
import wind.android.market.business.option.model.OptionMonthModel
import wind.android.market.business.option.model.OptionTitleSpeedModel
import wind.android.market.business.option.view.IOptionDetailView
import wind.android.news.anews.StockUtil
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:      OptionDetailPresenter$
 * # Function:       期权详情页-数据控制器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2019/12/25$    15:56$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class OptionDetailPresenter(val mWindCode: String, val mView: IOptionDetailView): ActivityHandler.ActivityHandlerListener,
        ISkyDataListenerEx{

    companion object {
        /**
         * 设置标题
         */
        const val MSG_SET_OPTION_TITLE = 1001

        /**
         * 更新标题
         */
        const val MSG_UPDATE_OPTION_TITLE = 1002

        /**
         * 更新月份
         */
        const val MSG_UPDATE_OPTION_MONTH = 1003
    }

    private val mManager by lazy { OptionManager() }

    private var mMonthRequestId = -3

    /**
     * 标题订阅所需指标
     */
    private val mTitleIndicator = intArrayOf(Indicator.DI_NEWPRICE,
            Indicator.DI_SHORTNAMECHS,
            Indicator.DI_CHANGE,
            Indicator.DI_CHANGERANGE,
            Indicator.DI_PRICEUNIT)

    /**
     * 标题订阅接收器
     */
    private val mTitleSpeedReceiver = object : ITcpDataReceiver {

        override fun onErrorReceived(model: SpeedErrorModel?, type: Int) {
        }

        override fun getTimeOutValue(): Int {
            return 0
        }

        override fun onSubDataRecived(items: Any?) {
            val vector = items as Vector<*>
            if (vector.size == 0 || vector[0] !is RealQuoteItem) {
                return
            }
            val  rt = vector[0] as RealQuoteItem
            if (rt.WindCode.isNullOrEmpty()) {
                return
            }
            if (rt.WindCode == mWindCode && rt.indicators != null) {
                val msg = Message()
                msg.what = MSG_UPDATE_OPTION_TITLE
                msg.obj = rt
                sendMessage(msg)
            }
        }

        override fun onMaskDataReceived(requestID: Int, data: Any?, type: Int): Boolean {
            if (type == SpeedConst.REQ_MASKED_SUBUNSUB.toInt() && data is Vector<*>) {
                val vector = data as? Vector<*>
                if (vector == null || vector.size == 0 || vector[0] !is RealQuoteItem) {
                    return false
                }
                val  rt = vector[0] as RealQuoteItem
                if (rt.WindCode.isNullOrEmpty()) {
                    return false
                }
                if (rt.WindCode == mWindCode && rt.indicators != null) {
                    dealTitleData(rt)
                    return true
                }
            }
            return false
        }
    }

    fun onResume() {
        subTitle()
    }

    fun onPause() {
        unSubTitle()
    }

    /**
     * 请求月份信息
     */
    fun requestMonth(contractCode: String, exchange: String) {
        mMonthRequestId = mManager.requestOptionMonth(contractCode, exchange, this)
    }

    /**
     * 订阅期权标题数据
     */
    fun subTitle() {
        if (!mWindCode.isEmpty()) {
            TcpProcessor.getInstance().registerPushDataReceiver(mTitleSpeedReceiver)
            Stock.SubscribeRequest(arrayOf(mWindCode), null, mTitleIndicator, mTitleSpeedReceiver)
        }
    }

    /**
     * 取消订阅
     */
    fun unSubTitle() {
        if (!mWindCode.isEmpty()) {
            TcpProcessor.getInstance().unregisterPushDataReceiver(mTitleSpeedReceiver)
            Stock.SubscribeRequest(null, arrayOf(mWindCode), mTitleIndicator, mTitleSpeedReceiver)
        }
    }

    private fun dealTitleData(item: RealQuoteItem) {
        val model = OptionTitleSpeedModel(item.WindCode)
        parseTitle(item, model)
        val msg = Message()
        msg.what = MSG_SET_OPTION_TITLE
        msg.obj = model
        sendMessage(msg)
    }

    /**
     * 数据解析
     */
    fun parseTitle(item: RealQuoteItem, model: OptionTitleSpeedModel?) {
        model?.let {
            val indicators = item.indicators
            val size = indicators.size
            val precision = CommonFunc.getRadixPointFactor(item.WindCode)
            for (i in 0 until size) {
                when(indicators[i]) {
                    Indicator.DI_NEWPRICE -> {
                        it.mNewPrice = CommonFunc.floatFormat(item.value[i].toDouble(), precision)
                    }
                    Indicator.DI_CHANGE -> {
                        it.mChange = CommonFunc.floatFormat(item.value[i].toDouble(), precision)
                        it.mTextColor = StockUtil.getChangeColor(item.value[i])
                    }
                    Indicator.DI_CHANGERANGE -> {
                        it.mChangeRange = CommonFunc.fixTText(item.value[i].toDouble(), 2) + "%"
                    }
                }
            }
        }
    }

    private fun dealMonth(source: MutableList<MutableList<String?>>?) {
        val dataList = mutableListOf<OptionMonthModel>()
        source?.forEach {
            if (it.size == 3) {
                val model = OptionMonthModel()
                model.mContractCode = it[0] ?: ""
                model.mDate = it[1] ?: ""
                model.mDateStr = Assist.getDateByFormat(it[2] ?: "", "yyyy-MM-dd", "yyyy年MM月")
                dataList.add(model)
            }
        }
        dataList.sortBy { it.mDate }
        val msg = Message()
        msg.what = MSG_UPDATE_OPTION_MONTH
        msg.obj = dataList
        sendMessage(msg)
    }

    override fun OnSkyError(code: Int, serialNum: Int) {

    }

    override fun OnSkyCallback(data: SkyCallbackData?) {
        data?.data?.run {
            val serialNum = data.SerialNum
            if (mMonthRequestId == serialNum) {
                val source = this as? MutableList<MutableList<String?>>
                dealMonth(source)
            }
        }
    }

    private fun sendMessage(msg: Message) {
        ActivityHandler.getInstance(this).sendMessage(msg)
    }

    override fun handleMessage(msg: Message?) {
        msg?.let {
            when(it.what) {
                MSG_SET_OPTION_TITLE -> {
                    if (it.obj is OptionTitleSpeedModel) {
                        val model = msg.obj as OptionTitleSpeedModel
                        mView.setTitle(model)
                    }
                }
                MSG_UPDATE_OPTION_TITLE -> {
                    if (it.obj is RealQuoteItem) {
                        val item = it.obj as RealQuoteItem
                        mView.updateTitle(item)
                    }
                }
                MSG_UPDATE_OPTION_MONTH -> {
                    val data = msg.obj as? MutableList<OptionMonthModel>
                    if (data != null) {
                        mView.updateMonth(data)
                    }
                }
            }
        }
    }
}