package wind.android.market.business.option.presenter

import android.annotation.SuppressLint
import android.os.Handler
import android.os.Message
import net.bussiness.Stock
import net.data.network.SkyCallbackData
import net.datamodel.network.Indicator
import net.datamodel.network.RealQuoteItem
import net.datamodel.network.SpeedConst
import net.listener.ISkyDataListenerEx
import net.listener.ITcpDataReceiver
import net.network.model.SpeedErrorModel
import net.network.speed.TcpProcessor
import wind.android.market.business.option.manager.OptionManager
import wind.android.market.business.option.model.OptionContentModel
import wind.android.market.business.option.model.OptionDetailModel
import wind.android.market.business.option.model.OptionMonthModel
import wind.android.market.business.option.view.IOptionListView
import java.util.*

/**
 * # ***********************************************************************************************
 * # ClassName:      OptionListViewPresenter$
 * # Function:       期权列表视图-控制器
 * # @author         ybxie$
 * # @version        Ver 1.0
 * # ***********************************************************************************************
 * # Modified By     ybxie$     2020/1/3$    14:00$
 * # Modifications:  initial
 * # ***********************************************************************************************
 */
class OptionListViewPresenter(val mView: IOptionListView): ISkyDataListenerEx {

    companion object {
        /**
         * 设置期权列表
         */
        const val MSG_SET_OPTION_LIST = 1001

        /**
         * 初次更新期权列表视图
         */
        const val MSG_UPDATE_OPTION_LIST = 1002

        /**
         * 更新期权列表数据
         */
        const val MSG_UPDATE_OPTION_ITEM = 1003

        /**
         * 更新期权列表视图
         */
        const val MSG_REFRESH_OPTION_LIST = 1004
    }

    private val mHandler = @SuppressLint("HandlerLeak")
    object: Handler() {
        override fun handleMessage(msg: Message?) {
            msg?.let {
                when(it.what) {
                    MSG_SET_OPTION_LIST -> {
                        val data = msg.obj as? MutableList<OptionContentModel>
                        if (data != null) {
                            mView.setOptionList(data)
                        }
                    }
                    MSG_UPDATE_OPTION_LIST -> {
                        val data = msg.obj as? MutableList<RealQuoteItem>
                        if (data != null) {
                            mView.updateOptionList(data)
                        }
                    }
                    MSG_UPDATE_OPTION_ITEM -> {
                        val data = msg.obj as? RealQuoteItem
                        if (data != null) {
                            mView.updateOptionItem(data)
                        }
                        if (!hasMessages(MSG_REFRESH_OPTION_LIST)) {
                            sendEmptyMessageDelayed(MSG_REFRESH_OPTION_LIST, 1000)
                        }
                    }
                    MSG_REFRESH_OPTION_LIST -> {
                        mView.refreshOptionList()
                    }
                }
            }
        }
    }

    private var mContentRequestId = -4

    private val mManager by lazy { OptionManager() }

    /**
     * 期权列表订阅指标
     */
    private val mContentIndicator = intArrayOf(
            Indicator.DI_NEWPRICE,
            Indicator.DI_CHANGE,
            Indicator.DI_CHANGERANGE,
            Indicator.DI_HIDDING_VOLATILITY,
            Indicator.DI_DELTA_VALUE,
            Indicator.DI_GAMMA,
            Indicator.DI_VEGA_VALUE,
            Indicator.DI_THETA_VALUE,
            Indicator.DI_BUYPRICE1,
            Indicator.DI_SELLPRICE1,
            Indicator.DI_VOLUMN,
            Indicator.DI_TOTALAMOUNT,
            Indicator.DI_POSITION,
            Indicator.DI_ACTUAL_LEVEL_TIMES,
            Indicator.DI_INNER_PRICE,
            Indicator.DI_TIME_PRICE,
            Indicator.DI_PRICEUNIT)

    private var mSubList = mutableListOf<String>()

    /**
     * 期权列表订阅接收器
     */
    private val mContentSpeedReceiver = object : ITcpDataReceiver {

        override fun onErrorReceived(model: SpeedErrorModel?, type: Int) {
        }

        override fun getTimeOutValue(): Int {
            return 0
        }

        override fun onSubDataRecived(items: Any?) {
            if (items == null) {
                return
            }
            val vector = items as? Vector<*>
            if (vector == null || vector.size == 0 ) {
                return
            }
            val any = vector[0]
            if (any is RealQuoteItem) {
                val msg = Message()
                msg.what = MSG_UPDATE_OPTION_ITEM
                msg.obj = any
                sendMessage(msg)
            }
        }

        override fun onMaskDataReceived(requestID: Int, data: Any?, type: Int): Boolean {
            if (data == null) {
                return false
            }
            if (type == SpeedConst.REQ_MASKED_SUBUNSUB.toInt() && data is Vector<*>) {
                val vector = data as? Vector<RealQuoteItem>
                if (vector == null || vector.size == 0 ) {
                    return false
                }
                val msg = Message()
                msg.what = MSG_UPDATE_OPTION_LIST
                msg.obj = vector
                sendMessage(msg)
            }
            return false
        }
    }

    fun hasSubData(): Boolean {
        if (mSubList.size > 0) {
            return true
        }
        return false
    }

    /**
     * 请求期权列表
     */
    fun requestOptionContent(model: OptionMonthModel, exchange: String) {
        if (mSubList.isEmpty()) {
            mContentRequestId = mManager.requestOptionContent(model.mContractCode, exchange, model.mDate, this)
        } else {
            unSubOptionList()
            subOptionList()
        }
    }

    /**
     * 订阅期权列表
     */
    fun subOptionList() {
        if (mSubList.isEmpty()) {
            return
        }
        TcpProcessor.getInstance().registerPushDataReceiver(mContentSpeedReceiver)
        Stock.SubscribeRequest(mSubList.toTypedArray(), null, mContentIndicator, mContentSpeedReceiver)
    }

    /**
     * 取消订阅
     */
    fun unSubOptionList() {
        mHandler.removeMessages(MSG_UPDATE_OPTION_ITEM)
        mHandler.removeMessages(MSG_UPDATE_OPTION_LIST)
        mHandler.removeMessages(MSG_SET_OPTION_LIST)
        if (mSubList.isEmpty()) {
            return
        }
        TcpProcessor.getInstance().unregisterPushDataReceiver(mContentSpeedReceiver)
        Stock.SubscribeRequest(null, mSubList.toTypedArray(), mContentIndicator, mContentSpeedReceiver)
    }

    private fun dealOptionDetail(source: MutableList<MutableList<String?>>?) {
        val dataList = mutableListOf<OptionContentModel>()
        val leftList = mutableListOf<OptionDetailModel>()
        val rightList = mutableListOf<OptionDetailModel>()
        source?.forEach {
            val detailModel = OptionDetailModel()
            if (it.size == 8) {
                detailModel.windCode = it[0] ?: ""
                detailModel.contractsCode = it[1] ?: ""
                detailModel.exName = it[2] ?: ""
                detailModel.month = it[3] ?: ""
                detailModel.strikePrice = it[4] ?: ""
                detailModel.direction = it[5] ?: ""
                detailModel.contractUnit = it[6] ?: ""
                detailModel.multiplier = it[7] ?: ""
                if (detailModel.direction == "708001000") {
                    // 认购
                    leftList.add(detailModel)
                } else {
                    // 认沽
                    rightList.add(detailModel)
                }
            }
        }
        val codeList = mutableListOf<String>()
        leftList.forEach {
            leftModel ->
            rightList.forEach rightEach@{
                if (!leftModel.strikePrice.isEmpty() && leftModel.strikePrice == it.strikePrice) {
                    val model = OptionContentModel(leftModel.strikePrice, leftModel, it)
                    dataList.add(model)
                    if (!it.windCode.isEmpty()) {
                        codeList.add(it.windCode)
                    }
                    return@rightEach
                }
            }
        }
        dataList.sortBy { it.strikePrice }
        doSubOption(dataList)
        val msg = Message()
        msg.what = MSG_SET_OPTION_LIST
        msg.obj = dataList
        sendMessage(msg)
    }

    private fun doSubOption(list: MutableList<OptionContentModel>) {
        unSubOptionList()
        mSubList.clear()
        list.forEach {
            if (!it.leftModel.windCode.isEmpty()) {
                mSubList.add(it.leftModel.windCode)
            }
            if (!it.rightModel.windCode.isEmpty()) {
                mSubList.add(it.rightModel.windCode)
            }
        }
        subOptionList()
    }

    private fun sendMessage(msg: Message) {
        mHandler.sendMessage(msg)
    }

    override fun OnSkyError(code: Int, serialNum: Int) {

    }

    override fun OnSkyCallback(data: SkyCallbackData?) {
        data?.data?.run {
            val serialNum = data.SerialNum
            if (mContentRequestId == serialNum) {
                val source = this as MutableList<MutableList<String?>>
                dealOptionDetail(source)
            }
        }
    }

}